/*
raw2wav - contruct RIFF/X-Wave header and simply attaches an expected PCM-stream to it
Copyright (C) 2005 Sascha Retzki.
released under the terms of the MIT-License.
*/



#include <u.h>
#include <libc.h>

#include <libaudioconv.h>



void
usage(void)
{
	print("usage: %s [-a audioformat] [-c channels] [-s sample_rate] [-b bitspersample] [-B blockalign] [-t RIFF|RIFX] [-H] file.raw\n",argv0);

	print("-a\t\tSets the audioformat. Default: 1\n");

	print("-c\t\tSets the number of Channels. Default: 2\n");

	print("-s\t\tSets the sample rate. Default: 44100\n");

	print("-b\t\tSets Bits-Per-Sample. Default: 16\n");

	print("-B\t\tSets block alignment. Default: 4\n");

	print("-t\t\tHeader Type. Default: RIFF. Affects byte-order of FMT-chunk-values.\n");

	print("-H\t\tHeaderHack. Set filesize values in header randomly, read data from stdin, and pray everything works\n");

	print("elsewise, data is expected in a file (on which we must stat!) and passed to stdout\n");

	exits("usage");
}


void
main(int argc,char **argv)
{
	ACriffwave_fmt *fmt = calloc(1,sizeof(ACriffwave_fmt));
	Dir *filestat = malloc(sizeof(Dir));
	int filesize, headertype, headerhack, rr,  fd = 0 ; // fd explicitely set to stdin!
	char *data, *filename;

	USED(filestat);
	USED(headertype);

	ARGBEGIN {
		case 'a':
			if((fmt->audioformat = atoi(EARGF(usage()))) == 0)
				sysfatal("ASCII to Integer-convertion of argument to -a failed");
			break;
		case 'c':
			if((fmt->channels = atoi(EARGF(usage()))) == 0)
				sysfatal("ASCII to Integer-convertion of argument to -c failed");
			break;
		case 's':
			if((fmt->samplerate = atoi(EARGF(usage()))) == 0)
				sysfatal("ASCII to Integer-convertion of argument to -s failed");
			break;
		case 'b':
			if((fmt->bitspersample = (atoi(EARGF(usage()))/8)) == 0)
				sysfatal("ASCII to Integer-convertion of argument to -b failed");
			break;
		case 'B':
			if((fmt->blockalign = (atoi(EARGF(usage()))/8)) == 0)
				sysfatal("ASCII to Integer-convertion of argument to -B failed");
			break;
		case 'h':
			if(strcmp(EARGF(usage()),"RIFF") == 0)
				headertype = ACRIFF;
			else if(strcmp(EARGF(usage()), "RIFX") == 0)
				headertype = ACRIFX;
			else
				sysfatal("headertype unknown");
			break;
		case 'H':
			filesize = 1337;
			headerhack = 1;
			break;
		default:
			usage();
	} ARGEND



	if(!headerhack) {

		/* if the user wants us to hack, be bitchy about file.raw */
		switch(argc) {
			case 0:
				fprint(2,"you forgot to give me a file.\n");
				usage();
				break;
			case 1:
				filename = *argv;
				break;
			default:
				fprint(2,"sorry, what shall I do?\n");
				usage();
				break;
		}


		/* If the user wants us to hack, we stat. */

		if((filestat = dirstat(filename)) == 0)
			sysfatal("dirstat() failed: %r");

			filesize = filestat->length;
			if(filestat->mode & DMDIR)
				sysfatal("Specified file is a directory. How's that supposed to work?");
			free(filestat);
	}

	/* Check if there are all the values if not, put in defaults */
	if(fmt->audioformat == 0)
		fmt->audioformat = 1;
	if(fmt->channels == 0)
		fmt->channels = 2;
	if(fmt->samplerate == 0)
		fmt->samplerate = 44100;
	if(fmt->bitspersample == 0)
		fmt->bitspersample = 16;
	if(fmt->blockalign == 0)
		fmt->blockalign = 4;

	/* Don't forget to make up the byterate.. */
	fmt->byterate = fmt->samplerate * fmt->channels * (fmt->bitspersample/8);




	/* We've got everything, construct a RIFF/RIFX-WAVE header. Here we need the filesize :/ */
	write(1,mkriffxwave(headertype, filesize),ACHSRIFFWAVE);


	/* The FMT-chunk comes next  */
	write(1, mkfmt(headertype, fmt), ACHSFMT);


	/* Okay, we also need to make up a 'DATA'-chunk, even tho 99% of the work is while(read()) write() ;) */
	write(1,mkdata(filesize),ACHSDATA);



	/* You remember headerhack? Here we go. Note that fd has been set to 0 at declaration time for a reason... */
	if(!headerhack) {
		if((fd = open(filename,OREAD)) == 0)
			sysfatal("open on file failed: %r");
	}

	if((data = malloc(32768)) == 0)
		sysfatal("malloc %r");
		

	/* Okay so now we just write the PCM-data to stdout. */
	while((rr = read(fd, data, 32768)) != 0)
		write(1, data, rr);

	exits(nil);
}