/*
Libaudioconv - Audio converter functions
Copyright (C) 2005 Sascha Retzki.
released under the terms of the MIT-License.

Current implementation:
- Convert sample-rate, channels and bits
- Parse RIFF/RIFX - WAVE


*/


#pragma	lib	"libaudioconv.a"
#pragma	src	"/usr/glenda/libaudioconv/"


enum
{
	/* Byte-order: RIFF is little endian, RIFX is big endian. */
	ACRIFF,
	ACRIFX,

	/* Header-sizes */
	ACHSRIFFWAVE = 12,
	ACHSFMT = 24,
	ACHSDATA = 8,

	/* Error codes */
	ACError,				// unknown or generic error
	ACEaudioformat,		// Audioformat not supported. So, no PCM
	ACEchannels,			// channels claimed to be <= 0 or not supported 
	ACEsamplerate,		// samplerate claimed to be <= 0
	ACEbitspersample,		// bits per sample does not divide by 8
	ACEbyterate			// Byterate claimed to be != samplerate * channels * (bitspersample/8)
};


typedef struct ACriffwave_fmt {
	/* Tells us which codec is used (PCM = 1) */
	u16int audioformat;

	/* number of channels */
	u16int channels;

	/* how much samples per second */
	u32int samplerate;

	/* How much bytes per second (samplerate * channels * (bitspersample/8) )  */
	u32int byterate;

	/* Ignored */
	u16int blockalign;

	/* How much bits pre sample */
	u16int bitspersample;
}ACriffwave_fmt;



/* Functions: riffxwave.c */
int parseriffwave(unsigned char *);
int parsefmt(unsigned char *, int, ACriffwave_fmt *);
char *mkriffxwave(int, int);
unsigned char *mkfmt(int, ACriffwave_fmt *);
char *mkdata(int);


/* Functions: convert.c */
void convertsamplerate(short *, short *, long, long, long);
void convertmono2stereo(short *, short *, long);
void convert8bitspersample(char *, short *, long);
